@@ -56,13 +56,15 @@ class IBKR_AssetType:
 class IBKR_CorpActionType:
     NotSupported = -1
     _corporate_action_types = {
-        'TC': FOF.ACTION_MERGER,
-        'SO': FOF.ACTION_SPINOFF,
-        'IC': FOF.ACTION_SYMBOL_CHANGE,
-        'HI': FOF.ACTION_STOCK_DIVIDEND,
-        'FS': FOF.ACTION_SPLIT,
-        'RS': FOF.ACTION_SPLIT,
-        'BM': FOF.ACTION_BOND_MATURITY    # No separate value as will be converte to ordinary bond sell operation
+        'BM': FOF.ACTION_BOND_MATURITY,    # No separate value as will be converted to ordinary bond sell operation
+        'FS': FOF.ACTION_SPLIT,            # Forward split
+        'HI': FOF.PAYMENT_STOCK_DIVIDEND,  # Choice dividend
+        'IC': FOF.ACTION_SYMBOL_CHANGE,    # Issue change
+        'RS': FOF.ACTION_SPLIT,            # Reverse split
+        'SO': FOF.ACTION_SPINOFF,          # Spin-off of new company
+        'SD': FOF.PAYMENT_STOCK_DIVIDEND,  # Dividend paid in stocks
+        'TC': FOF.ACTION_MERGER,           # Conversion of one stock into another
+        'TO': FOF.ACTION_MERGER            # Voluntary conversion of one asset into another
     }
 
     def __init__(self, action_type):
@@ -111,11 +113,13 @@ class IBKR_Asset:
                 logging.warning(self.tr("Asset type isn't supported: ") + f"'{category}' ({symbol})")
             return
         self.id = max([0] + [x['id'] for x in assets_list]) + 1
-        asset = {"id": self.id, "symbol": symbol, 'name': name, 'type': category, 'exchange': exchange}
+        asset = {"id": self.id, "symbol": symbol, 'name': name, 'type': category}
         if isin:
             asset['isin'] = isin
         if cusip:
             asset['reg_code'] = cusip
+        if exchange and exchange != "VALUE":   # store exchange only if it is valuable
+            asset['exchange'] = exchange
         assets_list.append(asset)
 
     def tr(self, text):
@@ -259,6 +263,7 @@ class StatementIBKR(StatementXML):
                                             ('transactionID', 'number', str, ''),
                                             ('description', 'description', str, None),
                                             ('quantity', 'quantity', float, None),
+                                            ('value', 'value', float, None),
                                             ('proceeds', 'proceeds', float, None),
                                             ('code', 'code', str, '')],
                                  'loader': self.load_corporate_actions},
@@ -287,6 +292,14 @@ class StatementIBKR(StatementXML):
     def tr(self, text):
         return QApplication.translate("IBKR", text)
 
+    def validate_file_header_attributes(self, attributes):
+        if 'type' not in attributes:
+            raise Statement_ImportError(self.tr("Interactive Brokers report type not found"))
+        if attributes['type'] == "TCF":
+            raise Statement_ImportError(self.tr("You try to import Trade confimation report, not Activity report"))
+        if attributes['type'] != 'AF':
+            raise Statement_ImportError(self.tr("Unknown Interactive Brokers report type: ") + f"{attributes['type']}")
+
     # Convert attribute 'attr_name' value into json open-format asset type
     @staticmethod
     def attr_asset_type(xml_element, attr_name, default_value):
@@ -491,15 +504,13 @@ class StatementIBKR(StatementXML):
             FOF.ACTION_MERGER: self.load_merger,
             FOF.ACTION_SPINOFF: self.load_spinoff,
             FOF.ACTION_SYMBOL_CHANGE: self.load_symbol_change,
-            FOF.ACTION_STOCK_DIVIDEND: self.load_stock_dividend,
+            FOF.PAYMENT_STOCK_DIVIDEND: self.load_stock_dividend,
             FOF.ACTION_SPLIT: self.load_split,
             FOF.ACTION_BOND_MATURITY: self.load_bond_maturity
         }
 
         cnt = 0
-        if any(action['code'] == StatementIBKR.CancelledFlag for action in actions):
-            actions = [action for action in actions if action['code'] != StatementIBKR.CancelledFlag]
-            logging.warning(self.tr("Statement contains cancelled corporate actions. They were skipped."))
+        self.remove_cancelled_corporate_actions(actions)
 
         # If stocks were bought/sold on a corporate action day IBKR may put several records for one corporate
         # action. So first step is to aggregate quantity.
@@ -540,10 +551,29 @@ class StatementIBKR(StatementXML):
             raise Statement_ImportError(self.tr("Can't find paired record for ") + f"{action}")
         return paired_record
 
+    # Takes cancelled corporate actions and tries to find and remove original one form actions list
+    def remove_cancelled_corporate_actions(self, actions):
+        delete_elements = []
+        cancelled_actions = [(idx, action) for idx, action in enumerate(actions) if action['code'] == StatementIBKR.CancelledFlag]
+        for c_id, c_action in cancelled_actions:
+            matched = [idx for (idx, action) in enumerate(actions) if
+                       action["type"] == c_action["type"] and action["account"] == c_action["account"] and
+                       action["asset"] == c_action["asset"] and action["timestamp"] == c_action["timestamp"] and
+                       action["description"] == c_action["description"] and action["quantity"] == -c_action["quantity"]]
+            if len(matched) == 1:
+                delete_elements += [c_id, matched[0]]
+        for idx in sorted(delete_elements, reverse=True):
+            del actions[idx]
+        for action in actions:
+            if action['code'] == StatementIBKR.CancelledFlag:
+                raise Statement_ImportError(self.tr("Can't process cancelled corporate action") + f" '{action}'")
+
     def load_merger(self, action, parts_b) -> int:
         MergerPatterns = [
             r"^(?P<symbol_old>\w+)(.OLD)?\((?P<isin_old>\w+)\) +MERGED\(\w+\) +WITH +(?P<isin_new>\w+) +(?P<X>\d+) +FOR +(?P<Y>\d+) +\((?P<symbol>\w+)(.OLD)?, (?P<name>.*), (?P<id>\w+)\)$",
-            r"^(?P<symbol_old>\w+)(.OLD)?\((?P<isin_old>\w+)\) +CASH and STOCK MERGER +\(\w+\) +(?P<isin_new>\w+) +(?P<X>\d+) +FOR +(?P<Y>\d+) +AND +(?P<currency>\w+) +(\d+(\.\d+)?) +\((?P<symbol>\w+)(.OLD)?, (?P<name>.*), (?P<id>\w+)\)$"
+            r"^(?P<symbol_old>\w+)(.OLD)?\((?P<isin_old>\w+)\) +CASH and STOCK MERGER +\(\w+\) +(?P<isin_new>\w+) +(?P<X>\d+) +FOR +(?P<Y>\d+) +AND +(?P<currency>\w+) +(\d+(\.\d+)?) +\((?P<symbol>\w+)(.OLD)?, (?P<name>.*), (?P<id>\w+)\)$",
+            r"^(?P<symbol_old>\w+)(.OLD)?\((?P<isin_old>\w+)\) +CASH and STOCK MERGER +\(\w+\) +(?P<isin_new>\w+) +(?P<X>\d+) +FOR +(?P<Y>\d+), +(?P<isin_new2>\w+) +(?P<X2>\d+) +FOR +(?P<Y2>\d+) +AND +(?P<currency>\w+) +(\d+(\.\d+)?) +\((?P<symbol>\w+)(.OLD)?, (?P<name>.*), (?P<id>\w+)\)$",
+            r"^(?P<symbol_old>.*)\((?P<isin_old>\w+)\) +TENDERED TO +(?P<isin_new>\w+) +(?P<X>\d+) +FOR +(?P<Y>\d+) +\((?P<symbol>.*), +(?P<name>.*), +(?P<id>.*)\)$"
             ]
 
         parts = None
@@ -560,14 +590,21 @@ class StatementIBKR(StatementXML):
         description_b = action['description'][:parts.span('symbol')[0]] + merger_a['symbol_old']
         asset_b = self.locate_asset(merger_a['symbol_old'], merger_a['isin_old'])
         paired_record = self.find_corp_action_pair(asset_b, description_b, action, parts_b)
-        if pattern_id == 1:
+        if (pattern_id == 1 or pattern_id == 2) and (not paired_record[0]['jal_processed']):
             self.add_merger_payment(action['timestamp'], action['account'], paired_record[0]['proceeds'],
                                     parts['currency'], action['description'])
+        if pattern_id == 2 and (not paired_record[0]['jal_processed']):
+            action['timestamp'] -= 1
+            action['type'] = FOF.ACTION_SPINOFF   # FIXME it is temporary workaround to keep 2 outgoing assets - one as from spin-off, another from merger
         action['id'] = max([0] + [x['id'] for x in self._data[FOF.CORP_ACTIONS]]) + 1
         action['cost_basis'] = 1.0
         action['asset'] = [paired_record[0]['asset'], action['asset']]
-        action['quantity'] = [-paired_record[0]['quantity'], action['quantity']]
-        self.drop_extra_fields(action, ["proceeds", "code", "asset_type", "jal_processed"])
+        if action['asset_type'] == FOF.ASSET_BOND:  # Adjust number of bonds
+            action['quantity'] = [-paired_record[0]['quantity']/IBKR_Asset.BondPrincipal,
+                                  action['quantity']/IBKR_Asset.BondPrincipal]
+        else:
+            action['quantity'] = [-paired_record[0]['quantity'], action['quantity']]
+        self.drop_extra_fields(action, ["value", "proceeds", "code", "asset_type", "jal_processed"])
         self._data[FOF.CORP_ACTIONS].append(action)
         paired_record[0]['jal_processed'] = True
         return 2
@@ -595,11 +632,14 @@ class StatementIBKR(StatementXML):
         if not asset_old:
             raise Statement_ImportError(self.tr("Spin-off initial asset not found ") + f"'{action}'")
         qty_old = int(spinoff['Y']) * action['quantity'] / int(spinoff['X'])
+        if abs(round(qty_old) - qty_old) > 0.01:
+            raise Statement_ImportError(self.tr("Spin-off rounding error is too big ") + f"'{action}'")
+        qty_old = round(qty_old)
         action['id'] = max([0] + [x['id'] for x in self._data[FOF.CORP_ACTIONS]]) + 1
         action['cost_basis'] = 0.0
         action['asset'] = [asset_old, action['asset']]
         action['quantity'] = [qty_old, action['quantity']]
-        self.drop_extra_fields(action, ["proceeds", "code", "asset_type", "jal_processed"])
+        self.drop_extra_fields(action, ["value", "proceeds", "code", "asset_type", "jal_processed"])
         self._data[FOF.CORP_ACTIONS].append(action)
         return 1
 
@@ -619,16 +659,25 @@ class StatementIBKR(StatementXML):
         action['cost_basis'] = 1.0
         action['asset'] = [paired_record[0]['asset'], action['asset']]
         action['quantity'] = [-paired_record[0]['quantity'], action['quantity']]
-        self.drop_extra_fields(action, ["proceeds", "code", "asset_type", "jal_processed"])
+        self.drop_extra_fields(action, ["value", "proceeds", "code", "asset_type", "jal_processed"])
         self._data[FOF.CORP_ACTIONS].append(action)
         paired_record[0]['jal_processed'] = True
         return 2
 
     def load_stock_dividend(self, action, parts_b) -> int:
-        action['id'] = max([0] + [x['id'] for x in self._data[FOF.CORP_ACTIONS]]) + 1
-        action['cost_basis'] = 0.0
-        self.drop_extra_fields(action, ["proceeds", "code", "asset_type", "jal_processed"])
-        self._data[FOF.CORP_ACTIONS].append(action)
+        StockDividendPattern = r"^(?P<description>.*) +(?P<tail>\(.*\))$"
+
+        parts = re.match(StockDividendPattern, action['description'], re.IGNORECASE)
+        if parts is None:
+            raise Statement_ImportError(self.tr("Can't parse Stock Dividend description ") + f"'{action}'")
+        action['description'] = parts.groupdict()['description']
+
+        action['id'] = max([0] + [x['id'] for x in self._data[FOF.ASSET_PAYMENTS]]) + 1
+        action['amount'] = action['quantity']
+        action['price'] = action['value'] / action['quantity']
+        action['tax'] = 0
+        self.drop_extra_fields(action, ["quantity", "value", "proceeds", "code", "asset_type", "jal_processed"])
+        self._data[FOF.ASSET_PAYMENTS].append(action)
         return 1
 
     def load_split(self, action, parts_b) -> int:
@@ -648,7 +697,7 @@ class StatementIBKR(StatementXML):
             action['cost_basis'] = 1.0
             action['asset'] = [action['asset'], action['asset']]
             action['quantity'] = [qty_old, qty_new]
-            self.drop_extra_fields(action, ["proceeds", "code", "asset_type", "jal_processed"])
+            self.drop_extra_fields(action, ["value", "proceeds", "code", "asset_type", "jal_processed"])
             self._data[FOF.CORP_ACTIONS].append(action)
             return 1
         else:  # Split together with ISIN change and there should be 2nd record available
@@ -659,7 +708,7 @@ class StatementIBKR(StatementXML):
             action['cost_basis'] = 1.0
             action['asset'] = [paired_record[0]['asset'], action['asset']]
             action['quantity'] = [-paired_record[0]['quantity'], action['quantity']]
-            self.drop_extra_fields(action, ["proceeds", "code", "asset_type", "jal_processed"])
+            self.drop_extra_fields(action, ["value", "proceeds", "code", "asset_type", "jal_processed"])
             self._data[FOF.CORP_ACTIONS].append(action)
             paired_record[0]['jal_processed'] = True
             return 2
@@ -672,7 +721,8 @@ class StatementIBKR(StatementXML):
         action['settlement'] = action['timestamp']                    # Settled by the same date
         action['note'] = action['description']
         action['fee'] = 0.0
-        self.drop_extra_fields(action, ["description", "proceeds", "type", "code", "asset_type", "jal_processed"])
+        self.drop_extra_fields(action, ["description", "value", "proceeds", "type", "code", "asset_type",
+                                        "jal_processed"])
         self._data[FOF.TRADES].append(action)
         return 1
 
@@ -788,7 +838,8 @@ class StatementIBKR(StatementXML):
         DividendNotePattern = r"^(?P<symbol>.*\w) ?\((?P<isin>\w+)\)(?P<prefix>( \w*)+) +(?P<amount>\d+\.\d+)?(?P<suffix>.*) \(.*\)$"
 
         dividends = [x for x in self._data[FOF.ASSET_PAYMENTS] if
-                     x['type'] == FOF.PAYMENT_DIVIDEND and x['asset'] == asset_id and x['account'] == account_id]
+                     (x['type'] == FOF.PAYMENT_DIVIDEND or x['type'] == FOF.PAYMENT_STOCK_DIVIDEND)
+                     and x['asset'] == asset_id and x['account'] == account_id]
         account = [x for x in self._data[FOF.ACCOUNTS] if x["id"] == account_id][0]
         currency = [x for x in self._data[FOF.ASSETS] if x["id"] == account['currency']][0]
         db_account = JalDB().get_account_id(account['number'], currency['symbol'])
