@@ -202,6 +202,10 @@ class JalDB:
             _ = executeSQL("UPDATE assets SET expiry=:expiry WHERE id=:asset_id",
                            [(":expiry", expiry), (":asset_id", asset_id)])
 
+    def get_quote(self, asset_id, timestamp):
+        return readSQL("SELECT quote FROM quotes WHERE asset_id=:asset_id AND timestamp=:timestamp",
+                       [(":asset_id", asset_id), (":timestamp", timestamp)])
+
     def update_quote(self, asset_id, timestamp, quote):
         if (timestamp is None) or (quote is None):
             return
@@ -233,11 +237,12 @@ class JalDB:
                            [(":new_reg", reg_code), (":asset_id", asset_id)])
         return asset_id
 
-    def add_dividend(self, subtype, timestamp, account_id, asset_id, amount, note, trade_number='', tax=0.0):
-        id = readSQL("SELECT id FROM dividends WHERE timestamp=:timestamp AND account_id=:account_id "
+    def add_dividend(self, subtype, timestamp, account_id, asset_id, amount, note, trade_number='',
+                     tax=0.0, price=None):
+        id = readSQL("SELECT id FROM dividends WHERE timestamp=:timestamp AND type=:subtype AND account_id=:account_id "
                      "AND asset_id=:asset_id AND amount=:amount AND note=:note",
-                     [(":timestamp", timestamp), (":account_id", account_id), (":asset_id", asset_id),
-                      (":amount", amount), (":note", note)])
+                     [(":timestamp", timestamp), (":subtype", subtype), (":account_id", account_id),
+                      (":asset_id", asset_id), (":amount", amount), (":note", note)])
         if id:
             logging.info(self.tr("Dividend already exists: ") + f"{note}")
             return
@@ -247,6 +252,8 @@ class JalDB:
                         (":account_id", account_id), (":asset_id", asset_id), (":amount", amount),
                         (":tax", tax), (":note", note)],
                        commit=True)
+        if price is not None:
+            self.update_quote(asset_id, timestamp, price)
 
     def update_dividend_tax(self, dividend_id, new_tax):
         _ = executeSQL("UPDATE dividends SET tax=:tax WHERE id=:dividend_id",
