@@ -183,6 +183,13 @@ class Ledger(QObject):
             self.processActionDetails()
 
     def processDividend(self):
+        if not self.current['subtype'] in [v for n, v in vars(DividendSubtype).items() if not n.startswith('_')]:
+            raise ValueError(self.tr("Unsupported dividend type.") + f" Operation: {self.current}")
+
+        if self.current['subtype'] == DividendSubtype.StockDividend:
+            self.processStockDividend()
+            return
+
         if self.current['subtype'] == DividendSubtype.Dividend:
             self.current['category'] = PredefinedCategory.Dividends
         elif self.current['subtype'] == DividendSubtype.BondInterest:
@@ -210,6 +217,28 @@ class Ledger(QObject):
             self.current['category'] = PredefinedCategory.Taxes
             self.appendTransaction(BookAccount.Costs, tax_amount)
 
+    def processStockDividend(self):
+        asset_id = self.current['asset']
+        qty = self.current['amount']
+        tax_amount = self.current['fee_tax']
+        asset_amount = self.getAmount(BookAccount.Assets, asset_id)
+        if asset_amount < -Setup.CALC_TOLERANCE:
+            raise NotImplemented(self.tr("Not supported action: stock dividend closes short trade.") +
+                                 f" Operation: {self.current}")
+        quote = JalDB().get_quote(asset_id, self.current['timestamp'])
+        if quote is None:
+            raise ValueError(self.tr("No stock quote for stock dividend.") + f" Operation: {self.current}")
+        _ = executeSQL("INSERT INTO open_trades(timestamp, op_type, operation_id, account_id, asset_id, price, remaining_qty) "
+                       "VALUES(:timestamp, :type, :operation_id, :account_id, :asset_id, :price, :remaining_qty)",
+                       [(":timestamp", self.current['timestamp']), (":type", TransactionType.Dividend),
+                        (":operation_id", self.current['id']), (":account_id", self.current['account']),
+                        (":asset_id", asset_id), (":price", quote), (":remaining_qty", qty)])
+        self.appendTransaction(BookAccount.Assets, qty, qty*quote)
+        if tax_amount:
+            self.appendTransaction(BookAccount.Money, -tax_amount)
+            self.current['category'] = PredefinedCategory.Taxes
+            self.appendTransaction(BookAccount.Costs, tax_amount)
+
     # Process buy or sell operation base on self.current['amount'] (>0 - buy, <0 - sell)
     def processTrade(self):
         if self.current['peer'] == '':
@@ -245,10 +274,15 @@ class Ledger(QObject):
                                "WHERE op_type=:op_type AND operation_id=:id AND asset_id=:asset_id",
                                [(":qty", next_deal_qty), (":op_type", opening_trade['op_type']),
                                 (":id", opening_trade['operation_id']), (":asset_id", asset_id)])
-                _ = executeSQL("INSERT INTO deals(account_id, asset_id, open_op_type, open_op_id, open_timestamp, close_op_type, close_op_id, close_timestamp, qty) "
-                               "VALUES(:account_id, :asset_id, :open_op_type, :open_op_id, :open_timestamp, :close_op_type, :close_op_id, :close_timestamp, :qty)",
-                               [(":account_id", account_id), (":asset_id", asset_id), (":open_op_type", opening_trade['op_type']), (":open_op_id", opening_trade['operation_id']), (":open_timestamp", opening_trade['timestamp']),
-                                (":close_op_type", TransactionType.Trade), (":close_op_id", self.current['id']), (":close_timestamp", self.current['timestamp']), (":qty", (-type)*next_deal_qty)])
+                _ = executeSQL("INSERT INTO deals(account_id, asset_id, open_op_type, open_op_id, open_timestamp, open_price, "
+                               "close_op_type, close_op_id, close_timestamp, close_price, qty) "
+                               "VALUES(:account_id, :asset_id, :open_op_type, :open_op_id, :open_timestamp, :open_price, "
+                               ":close_op_type, :close_op_id, :close_timestamp, :close_price, :qty)",
+                               [(":account_id", account_id), (":asset_id", asset_id),
+                                (":open_op_type", opening_trade['op_type']), (":open_op_id", opening_trade['operation_id']),
+                                (":open_timestamp", opening_trade['timestamp']), (":open_price", opening_trade['price']),
+                                (":close_op_type", TransactionType.Trade), (":close_op_id", self.current['id']),
+                                (":close_timestamp", self.current['timestamp']), (":close_price", price), (":qty", (-type)*next_deal_qty)])
                 processed_qty += next_deal_qty
                 processed_value += (next_deal_qty * opening_trade['price'])
                 if processed_qty == qty:
@@ -275,6 +309,9 @@ class Ledger(QObject):
             self.appendTransaction(BookAccount.Costs, self.current['fee_tax'])
 
     def processTransfer(self):
+        if not self.current['subtype'] in [v for n, v in vars(TransferSubtype).items() if not n.startswith('_')]:
+            raise ValueError(self.tr("Unsupported transfer type.") + f" Operation: {self.current}")
+
         if self.current['subtype'] == TransferSubtype.Outgoing:
             credit_taken = self.takeCredit(self.current['amount'])
             self.appendTransaction(BookAccount.Money, -(self.current['amount'] - credit_taken))
@@ -295,38 +332,21 @@ class Ledger(QObject):
             logging.error(self.tr("Unexpected data in transfer transaction"))
             return
 
-    def updateStockDividendAssets(self):
-        asset_amount = self.getAmount(BookAccount.Assets, self.current['asset'])
-        self.current['price'] = self.current['price'] + asset_amount
-        self.current['amount'] = asset_amount
-        asset = JalDB().get_asset_name(self.current['asset'])
-        QMessageBox().information(None, self.tr("Confirmation"),
-                                  self.tr("Stock dividend for was updated for ") + asset +
-                                  f" @{datetime.utcfromtimestamp(self.current['timestamp']).strftime('%d.%m.%Y')}\n" +
-                                  self.tr("Please check that quantity is correct."),
-                                  QMessageBox.Ok)
-        _ = executeSQL("UPDATE corp_actions SET qty=:qty, qty_new=:qty_new WHERE id=:id",
-                       [(":id", self.current['id']),
-                        (":qty", self.current['amount']), (":qty_new", self.current['price'])])
-
     def processCorporateAction(self):
-        # Stock dividends are imported without initial stock amounts -> correction happens here
-        if self.current['subtype'] == CorporateAction.StockDividend and self.current['amount'] < 0:
-            self.updateStockDividendAssets()
-
         account_id = self.current['account']
         asset_id = self.current['asset']
         qty = self.current['amount']
+        if not self.current['subtype'] in [v for n, v in vars(CorporateAction).items() if not n.startswith('_')]:
+            raise ValueError(self.tr("Unsupported corporate action type.") + f" Operation: {self.current}")
 
         processed_qty = 0
         processed_value = 0
         # Get asset amount accumulated before current operation
         asset_amount = self.getAmount(BookAccount.Assets, asset_id)
         if asset_amount < (qty - 2*Setup.CALC_TOLERANCE):
-            logging.fatal(self.tr("Asset amount is not enough for corporate action processing. Date: ")
-                          + f"{datetime.utcfromtimestamp(self.current['timestamp']).strftime('%d/%m/%Y %H:%M:%S')}")
-            return
-
+            raise ValueError(self.tr("Asset amount is not enough for corporate action processing. Date: ")
+                             + f"{datetime.utcfromtimestamp(self.current['timestamp']).strftime('%d/%m/%Y %H:%M:%S')}, "
+                             + f"Asset amount: {asset_amount}, Qty required: {qty}, Operation: {self.current}")
         # Get a list of all previous not matched trades or corporate actions
         query = executeSQL("SELECT timestamp, op_type, operation_id, account_id, asset_id, price, remaining_qty "
                            "FROM open_trades "
@@ -336,31 +356,39 @@ class Ledger(QObject):
         while query.next():
             opening_trade = readSQLrecord(query, named=True)
             next_deal_qty = opening_trade['remaining_qty']
-            if (processed_qty + next_deal_qty) > qty:  # We can't close all trades with current operation
-                raise ValueError(self.tr("Unhandled case: Corporate action covers not full open position"))
+            if (processed_qty + next_deal_qty) > (qty + 2*Setup.CALC_TOLERANCE):  # We can't close all trades with current operation
+                raise ValueError(self.tr("Unhandled case: Corporate action covers not full open position. Date: ")
+                                 + f"{datetime.utcfromtimestamp(self.current['timestamp']).strftime('%d/%m/%Y %H:%M:%S')}, "
+                                 + f"Processed: {processed_qty}, Next: {next_deal_qty}, Qty: {qty}, Operation: {self.current}")
             _ = executeSQL("UPDATE open_trades SET remaining_qty=0 "
                            "WHERE op_type=:op_type AND operation_id=:id AND asset_id=:asset_id",
-                           [(":op_type", opening_trade['op_type']),(":id", opening_trade['operation_id']),
+                           [(":op_type", opening_trade['op_type']), (":id", opening_trade['operation_id']),
                             (":asset_id", asset_id)])
 
-            # Create a deal with relevant sign of quantity (-1 for short, +1 for long)
-            _ = executeSQL("INSERT INTO deals(account_id, asset_id, open_op_type, open_op_id, open_timestamp, close_op_type, close_op_id, close_timestamp, qty) "
-                           "VALUES(:account_id, :asset_id, :open_op_type, :open_op_id, :open_timestamp, :close_op_type, :close_op_id, :close_timestamp, :qty)",
-                           [(":account_id", account_id), (":asset_id", asset_id), (":open_op_type", opening_trade['op_type']),(":open_op_id", opening_trade['operation_id']), (":open_timestamp", opening_trade['timestamp']),
-                            (":close_op_type", TransactionType.CorporateAction), (":close_op_id", self.current['id']), (":close_timestamp", self.current['timestamp']), (":qty", next_deal_qty)])
+            # Deal have the same open and close prices as corportate action doesn't create profit, but redistributes value
+            _ = executeSQL("INSERT INTO deals(account_id, asset_id, open_op_type, open_op_id, open_timestamp, open_price, "
+                           " close_op_type, close_op_id, close_timestamp, close_price, qty) "
+                           "VALUES(:account_id, :asset_id, :open_op_type, :open_op_id, :open_timestamp, :open_price, "
+                           ":close_op_type, :close_op_id, :close_timestamp, :close_price, :qty)",
+                           [(":account_id", account_id), (":asset_id", asset_id),
+                            (":open_op_type", opening_trade['op_type']), (":open_op_id", opening_trade['operation_id']),
+                            (":open_timestamp", opening_trade['timestamp']), (":open_price", opening_trade['price']),
+                            (":close_op_type", TransactionType.CorporateAction), (":close_op_id", self.current['id']),
+                            (":close_timestamp", self.current['timestamp']), (":close_price", opening_trade['price']),
+                            (":qty", next_deal_qty)])
             processed_qty += next_deal_qty
             processed_value += (next_deal_qty * opening_trade['price'])
             if processed_qty == qty:
                 break
         # Asset allocations for different corporate actions:
-        # +-----------------+-------+-----+------------+-----------+----------+---------------+--------------------+
-        # |                 | Asset | Qty | cost basis | Asset new | Qty new  | cost basis    | The same algo for: |
-        # +-----------------+-------+-----+------------+-----------+----------+---------------+--------------------+
-        # | Symbol Change   |   A   |  N  |  100 %     |     B     |    N     |   100%        |                    |
-        # | (R-)Split       |   A   |  N  |  100 %     |     A     |    M     |   100%        | Stock Dividend     |
-        # | Merger          |   A   |  N  |  100 %     |     B     |    M     |   100%        |                    |
-        # | Spin-Off        |   A   |  N  |  100 %     |   A & B   | AxN, BxM | X% & (100-X)% |                    |
-        # +-----------------+-------+-----+------------+-----------+----------+---------------+--------------------+
+        # +-----------------+-------+-----+------------+-----------+----------+---------------+
+        # |                 | Asset | Qty | cost basis | Asset new | Qty new  | cost basis    |
+        # +-----------------+-------+-----+------------+-----------+----------+---------------+
+        # | Symbol Change   |   A   |  N  |  100 %     |     B     |    N     |   100%        |
+        # | (R-)Split       |   A   |  N  |  100 %     |     A     |    M     |   100%        |
+        # | Merger          |   A   |  N  |  100 %     |     B     |    M     |   100%        |
+        # | Spin-Off        |   A   |  N  |  100 %     |   A & B   | AxN, BxM | X% & (100-X)% |
+        # +-----------------+-------+-----+------------+-----------+----------+---------------+
         # Withdraw value with old quantity of old asset as it common for all corporate action
         self.appendTransaction(BookAccount.Assets, -processed_qty, -processed_value)
 
@@ -404,6 +432,7 @@ class Ledger(QObject):
             TransactionType.CorporateAction: self.processCorporateAction
         }
 
+        exception_happened = False
         self.amounts.clear()
         self.values.clear()
         if from_timestamp >= 0:
@@ -446,10 +475,15 @@ class Ledger(QObject):
                 operationProcess[self.current['type']]()
                 if self.progress_bar is not None:
                     self.progress_bar.setValue(query.at())
+        except Exception as e:
+            exception_happened = True
+            logging.error(f"{e}")
         finally:
             if fast_and_dirty:
                 _ = executeSQL("PRAGMA synchronous = ON")
             db_triggers_enable()
+            if self.progress_bar is not None:
+                self.main_window.showProgressBar(False)
         # Fill ledger totals values
         _ = executeSQL("INSERT INTO ledger_totals"
                        "(op_type, operation_id, timestamp, book_account, asset_id, account_id, amount_acc, value_acc) "
@@ -458,12 +492,13 @@ class Ledger(QObject):
                        "WHERE id IN ("
                        "SELECT MAX(id) FROM ledger WHERE timestamp >= :frontier "
                        "GROUP BY op_type, operation_id, book_account, account_id)", [(":frontier", frontier)])
-
-        if self.progress_bar is not None:
-            self.main_window.showProgressBar(False)
         JalSettings().setValue('RebuildDB', 0)
-        logging.info(self.tr("Ledger is complete. Elapsed time: ") + f"{datetime.now() - start_time}" +
-                     self.tr(", new frontier: ") + f"{datetime.utcfromtimestamp(self.current['timestamp']).strftime('%d/%m/%Y %H:%M:%S')}")
+        if exception_happened:
+            logging.error(self.tr("Exception happened. Ledger is incomplete. Please correct errors listed in log"))
+        else:
+            logging.info(self.tr("Ledger is complete. Elapsed time: ") + f"{datetime.now() - start_time}" +
+                         self.tr(", new frontier: ") +
+                         f"{datetime.utcfromtimestamp(self.current['timestamp']).strftime('%d/%m/%Y %H:%M:%S')}")
 
         self.updated.emit()
 
