@@ -2,11 +2,11 @@ import logging
 
 import pandas as pd
 from PySide6.QtCore import Qt, QAbstractTableModel
-from PySide6.QtWidgets import QDialog
 from PySide6.QtGui import QFont
 from jal.db.helpers import executeSQL, readSQL, readSQLrecord
 from jal.db.db import JalDB
-from jal.ui.ui_tax_estimation import Ui_TaxEstimationDialog
+from jal.ui.reports.ui_tax_estimation import Ui_TaxEstimationDialog
+from jal.widgets.mdi import MdiWidget
 
 
 class TaxEstimatorModel(QAbstractTableModel):
@@ -54,8 +54,8 @@ class TaxEstimatorModel(QAbstractTableModel):
         return None
 
 
-class TaxEstimator(QDialog, Ui_TaxEstimationDialog):
-    def __init__(self, account_id, asset_id, asset_qty, position, parent=None):
+class TaxEstimator(MdiWidget, Ui_TaxEstimationDialog):
+    def __init__(self, account_id, asset_id, asset_qty, parent=None):
         super(TaxEstimator, self).__init__(parent)
         self.setupUi(self)
 
@@ -67,8 +67,6 @@ class TaxEstimator(QDialog, Ui_TaxEstimationDialog):
         self.ready = False
 
         self.setWindowTitle(self.tr("Tax estimation for ") + self.asset_name)
-        self.setWindowFlag(Qt.Tool)
-        self.setGeometry(position.x(), position.y(), self.width(), self.height())
 
         font = self.DealsView.horizontalHeader().font()
         font.setBold(True)
@@ -128,12 +126,11 @@ class TaxEstimator(QDialog, Ui_TaxEstimationDialog):
 
         query = executeSQL("SELECT strftime('%d/%m/%Y', datetime(t.timestamp, 'unixepoch')) AS timestamp, "
                            "t.qty AS qty, t.price AS o_price, oq.quote AS o_rate FROM trades AS t "
-                           "LEFT JOIN sequence AS s ON s.operation_id = t.id AND s.type = 3 "
                            "LEFT JOIN accounts AS ac ON ac.id = :account_id "
                            "LEFT JOIN t_last_dates AS od ON od.ref_id = IIF(t.settlement=0, t.timestamp, t.settlement) "
                            "LEFT JOIN quotes AS oq ON ac.currency_id=oq.asset_id AND oq.timestamp=od.timestamp "
                            "WHERE t.account_id=:account_id AND t.asset_id=:asset_id AND t.qty*(:total_qty)>0 "
-                           "ORDER BY s.id DESC",
+                           "ORDER BY t.timestamp DESC, t.id DESC",
                            [(":account_id", self.account_id), (":asset_id", self.asset_id),
                             (":total_qty", self.asset_qty)])
         table = []
